package hpdos.handler;

import hpdos.grpc.Ack;
import hpdos.grpc.Nack;
import hpdos.grpc.Request;
import hpdos.grpc.Response;
import hpdos.lib.StorageModel;
import hpdos.lib.StorageService;
import hpdos.message.MessageConstants;
import hpdos.message.ResponseBuilder;

public class IOHandler {
    StorageService storageService;
    private final boolean isMaster;
    public IOHandler(StorageService storageService, boolean isMaster) {
        this.storageService = storageService;
        this.isMaster = isMaster;
    }

    public Response create(Request request) {
        Ack ack = null; Nack nack = null;
        StorageModel blob = new StorageModel(
                MessageConstants.INIT_VERSION,
                request.getDataSize(),
                request.getKey(),
                request.getAccessType(),
                request.getClientID(),
                request.getValue());
        boolean status = storageService.create(request.getKey(), blob);
        if (status) {
            ack = ResponseBuilder.buildAck(MessageConstants.INIT_VERSION,
                    blob.getDataSize(),
                    blob.getKey(),
                    blob.getCrc(),
                    blob.getValue());
        }
        else {
            nack = ResponseBuilder.buildNack(MessageConstants.INVALID_VERSION,
                    request.getKey());
        }
        if (status)
            return ResponseBuilder.buildResponsePacket(
                    MessageConstants.METADATA_CREATE, MessageConstants.STATUS_OK,
                    ack, null);
        return ResponseBuilder.buildResponsePacket(
                MessageConstants.METADATA_CREATE, MessageConstants.STATUS_IO_WRITE_FAILED,
                null, nack);
    }

    public Response update(Request request) {
        return null;
    }

    public Response delete(Request request) {
        return null;

    }

    public Response read(Request request) {
        Ack ack = null; Nack nack = null;
        int status;
        // Only Metadata Master can serve shared object lookup requests
        if (!isMaster && request.getAccessType() == MessageConstants.METADATA_ACCESS_SHARED) {
            status = MessageConstants.STATUS_SERVER_NOT_MASTER;
            nack = ResponseBuilder.buildNack(MessageConstants.INVALID_VERSION,
                    request.getKey());
        } else {
            StorageModel blob = storageService.readByKey(request.getKey());
            if (blob == null) {
                status = MessageConstants.STATUS_KEY_NOT_FOUND;
                nack = ResponseBuilder.buildNack(MessageConstants.INVALID_VERSION,
                        request.getKey());

            } else if (request.getAccessType() == MessageConstants.METADATA_ACCESS_PRIVATE &&
                    !request.getClientID().equals(blob.getOwner())) {
                status = MessageConstants.STATUS_OWNER_MISMATCH;
                nack = ResponseBuilder.buildNack(MessageConstants.INVALID_VERSION,
                        request.getKey());
            } else {
                status = MessageConstants.STATUS_OK;
                ack = ResponseBuilder.buildAck(MessageConstants.INIT_VERSION,
                        blob.getDataSize(),
                        blob.getKey(),
                        blob.getCrc(),
                        blob.getValue());
            }
        }
        return ResponseBuilder.buildResponsePacket(
                MessageConstants.METADATA_CREATE, status,
                ack, nack);
    }
}
