package hpdos.handler;

import hpdos.grpc.*;
import hpdos.lib.ReplicationService;
import hpdos.message.RequestBuilder;
import hpdos.message.ResponseBuilder;
import io.grpc.stub.StreamObserver;

import java.util.ArrayList;
import java.util.concurrent.ExecutionException;

public class NetworkHandler extends NetworkServiceGrpc.NetworkServiceImplBase {
    private final IOHandler ioHandler;
    private final ReplicationService replicationService;
    public NetworkHandler(IOHandler ioHandler, ReplicationService replicationService) {
        this.ioHandler = ioHandler;
        this.replicationService = replicationService;
    }

    @Override
    public void readMetadata(Packet requestPacket,
                        StreamObserver<Packet> responseObserver) {
        System.out.println("Data received" + requestPacket.toString());
        for (Request request: requestPacket.getRequestList()) {
            Response response = ioHandler.read(request);
            Packet packet = ResponseBuilder.buildPacket(response);
            System.out.println(packet);
            responseObserver.onNext(packet);
        }

        responseObserver.onCompleted();
    }

    @Override
    public void createMetadata(Packet requestPacket, StreamObserver<Packet> responseObserver) {
        System.out.println("new create request " + requestPacket);
        for (Request request: requestPacket.getRequestList()) {
            ioHandler.create(request);
        }
        System.out.println("Added to local memory");
        ReplicationRequest replicationRequest = RequestBuilder.
                buildReplicationRequest(new ArrayList<>(requestPacket.getRequestList()));
        ReplicationResponse replicationResponse = null;
        try {
            System.out.println("starting replication");
            replicationResponse = replicationService.replicateMetadata(replicationRequest);
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
        }
        System.out.println("replication complete");
        Packet packet = ResponseBuilder.buildPacket(new ArrayList<>(replicationResponse.getResponseList()));
        System.out.println(packet);
        responseObserver.onNext(packet);
        responseObserver.onCompleted();
    }

    @Override
    public void updateMetadata(Packet request, StreamObserver<Packet> responseObserver) {
        super.updateMetadata(request, responseObserver);
    }

    @Override
    public void deleteMetadata(Packet request, StreamObserver<Packet> responseObserver) {
        super.deleteMetadata(request, responseObserver);
    }
}
